# -*- coding: utf-8 -*-

from sqlalchemy import *
from migrate import *
import migrate.changeset
from sqlalchemy.dialects.mysql.pyodbc import MySQLDialect
from documents.versions import dump

def getDocumentType(meta):
    documenttype = Table('documenttype', meta,
        Column('doctype', Integer, nullable=False, primary_key=True, autoincrement=False),    
        Column('name', String(100), nullable=False),
        Column('moneyfactor', SmallInteger , nullable=False),
        mysql_engine='InnoDB'
    )
    return documenttype

def getGoodsItemType(meta):
    goodsitemtype = Table('goodsitemtype', meta,
        Column('doctype', Integer, ForeignKey('documenttype.doctype', name='inventdoctypefk', ondelete = 'CASCADE', onupdate = 'CASCADE'), nullable=False,  primary_key=True, autoincrement=False),
        Column('reverseoperation', SmallInteger, nullable=False, primary_key=True, autoincrement=False), # 0 прямая операция, 1 обратная
        Column('name', String(100), nullable=False),
        Column('factor', SmallInteger , nullable=False),
        mysql_engine='InnoDB'
    )
    return goodsitemtype

def getMoneyItemType(meta):
    moneyitemtype = Table('moneyitemtype', meta,
        Column('doctype', Integer, ForeignKey('documenttype.doctype', name='moneydoctypefk', ondelete = 'CASCADE', onupdate = 'CASCADE'), nullable=False, primary_key=True, autoincrement=False),
        Column('reverseoperation', SmallInteger, nullable=False, primary_key=True, autoincrement=False),    
        Column('name', String(100), nullable=False),
        Column('factor', SmallInteger , nullable=False),
        mysql_engine='InnoDB'
    )
    return moneyitemtype

def getDocument(meta):    
    return  Table('document', meta, autoload=True)
def getGoodsItem(meta):    
    return  Table('goodsitem', meta, autoload=True)
def getStornoGoodsItem(meta):    
    return  Table('stornogoodsitem', meta, autoload=True)
def getMoneyItem(meta):    
    return  Table('moneyitem', meta, autoload=True)

def dropTriggers(migrate_engine):
    migrate_engine.connect().execute("DROP TRIGGER IF EXISTS `triggerAfterInsert`;")
    migrate_engine.connect().execute("DROP TRIGGER IF EXISTS `triggerAfterUpdate`;")
    migrate_engine.connect().execute("DROP TRIGGER IF EXISTS `triggerAfterDelete`;")

def createTriggers(migrate_engine):
    trigger_after_insert = """
CREATE TRIGGER triggerAfterInsert AFTER INSERT ON document
FOR EACH ROW BEGIN
    SET @factor:=(select moneyfactor FROM documenttype WHERE doctype = new.doctype);
    UPDATE workshift 
    SET sumsale = COALESCE(sumsale, 0) + if(new.doctype = 1, new.sumb, 0),
        sumgain = COALESCE(sumgain, 0) + if(new.doctype in (1, 2), new.sumb, 0),
        sumdrawer = COALESCE(sumdrawer, 0) + (new.sumcash * @factor)
    WHERE workshiftid = new.workshiftid AND new.closed = 1;
END
"""
    exit
    trigger_after_update = """
CREATE TRIGGER triggerAfterUpdate AFTER UPDATE ON document
FOR EACH ROW BEGIN    
    set @oldfactor:=(select moneyfactor FROM documenttype WHERE doctype = old.doctype);
    set @newfactor:=(select moneyfactor FROM documenttype WHERE doctype = new.doctype);
    set @oldsumb=0;
    set @oldsumcash=0;    
    set @newsumb=0;    
    set @newsumcash=0;   
    
    IF old.closed = 1
    THEN
       set @oldsumb=old.sumb;
       set @oldsumcash=@oldfactor * old.sumcash;
    END IF;
    
    IF new.closed = 1 
    THEN
       set @newsumb=new.sumb;
       set @newsumcash=@newfactor * new.sumcash;
    END IF;        
    
    UPDATE workshift 
    SET sumsale = COALESCE(sumsale, 0) + if(new.doctype = 1, -@oldsumb + @newsumb, 0),
        sumgain = COALESCE(sumgain, 0) + if(new.doctype in (1, 2), -@oldsumb + @newsumb, 0),
        sumdrawer = COALESCE(sumdrawer, 0) - @oldsumcash + @newsumcash
    WHERE workshiftid = new.workshiftid;
END
"""
    
    trigger_after_delete = """
CREATE TRIGGER triggerAfterDelete AFTER DELETE ON document
FOR EACH ROW BEGIN
    set @factor:=(select moneyfactor FROM documenttype WHERE doctype = old.doctype);    
    UPDATE workshift 
    SET sumsale = COALESCE(sumsale, 0) - if(old.doctype = 1, old.sumb, 0),
        sumgain = COALESCE(sumgain, 0) - if(old.doctype in (1, 2), old.sumb, 0),
        sumdrawer = COALESCE(sumdrawer, 0) - (old.sumcash * @factor)
    WHERE workshiftid = old.workshiftid AND old.closed = 1;       
END
"""
        
    migrate_engine.connect().execute(trigger_after_insert)
    migrate_engine.connect().execute(trigger_after_update)
    migrate_engine.connect().execute(trigger_after_delete)
    
def upgrade(migrate_engine):
    # Upgrade operations go here. Don't create your own engine; bind migrate_engine
    # to your metadata    
    meta = MetaData(bind = migrate_engine)
    dump.store(migrate_engine)
    moneyitemtype = getMoneyItemType(meta)
    goodsitemtype = getGoodsItemType(meta)
    documenttype = getDocumentType(meta)
    
    if 'documenttypefk' in [key.name for key in getDocument(meta).foreign_keys]:
        migrate.changeset.constraint.ForeignKeyConstraint([getDocument(meta).c.doctype], [documenttype.c.doctype], name = 'documenttypefk').drop()   
        
    if isinstance(migrate_engine.connect().dialect, MySQLDialect):
        dropTriggers(migrate_engine)
    if moneyitemtype.exists(meta.bind):
        moneyitemtype.drop()
    if goodsitemtype.exists(meta.bind):
        goodsitemtype.drop()
    if documenttype.exists(meta.bind):
        documenttype.drop()
    
    
    documenttype.create()
    goodsitemtype.create()
    moneyitemtype.create()

    documenttype.insert().values(doctype =  1, moneyfactor =  1, name = 'Чек продажи').execute()
    documenttype.insert().values(doctype =  2, moneyfactor = -1, name = 'Чек возврата').execute()
    documenttype.insert().values(doctype =  3, moneyfactor =  1, name = 'Акт внесения денег').execute()
    documenttype.insert().values(doctype =  4, moneyfactor = -1, name = 'Акт выема денег').execute()
    documenttype.insert().values(doctype = 13, moneyfactor =  1, name = 'Остаток денег на начало смены').execute()    
    documenttype.insert().values(doctype = 16, moneyfactor =  0, name = 'Инвентаризация').execute()
    documenttype.insert().values(doctype = 17, moneyfactor = -1, name = 'Возврат тары').execute()
    documenttype.insert().values(doctype = 18, moneyfactor =  0, name = 'Возврат поставщику').execute()
    
    goodsitemtype.insert().values(doctype = 1,  reverseoperation = 0, factor = -1, name = 'Продажа').execute()
    goodsitemtype.insert().values(doctype = 1,  reverseoperation = 1, factor =  1, name = 'Возврат в чеке продажи').execute()
    goodsitemtype.insert().values(doctype = 2,  reverseoperation = 0, factor =  1, name = 'Возврат').execute()
    goodsitemtype.insert().values(doctype = 16, reverseoperation = 0, factor =  0, name = 'Инвентаризация').execute()
    goodsitemtype.insert().values(doctype = 17, reverseoperation = 0, factor =  1, name = 'Возврат тары').execute()
    goodsitemtype.insert().values(doctype = 18, reverseoperation = 0, factor = -1, name = 'Возврат поставщику').execute()
    
    moneyitemtype.insert().values(doctype = 1,  reverseoperation = 0, factor =  1, name = 'Принято по продаже').execute()
    moneyitemtype.insert().values(doctype = 1,  reverseoperation = 1, factor = -1, name = 'Сдача по продаже').execute()
    moneyitemtype.insert().values(doctype = 2,  reverseoperation = 0, factor = -1, name = 'Выдано по возврату').execute()
    moneyitemtype.insert().values(doctype = 2,  reverseoperation = 1, factor =  1, name = 'Сдача по возврату').execute()
    moneyitemtype.insert().values(doctype = 3,  reverseoperation = 0, factor =  1, name = 'Внесено в кассу').execute()
    moneyitemtype.insert().values(doctype = 4,  reverseoperation = 0, factor = -1, name = 'Изъято из кассы').execute()
    moneyitemtype.insert().values(doctype = 13, reverseoperation = 0, factor =  1, name = 'Остаток на начало смены').execute()
    moneyitemtype.insert().values(doctype = 17, reverseoperation = 0, factor = -1, name = 'Выдано по возврату тары').execute()
    moneyitemtype.insert().values(doctype = 17, reverseoperation = 1, factor =  1, name = 'Сдача по возврату тары').execute()

    if not 'reverseoperation' in getGoodsItem(meta).columns: 
        Column('reverseoperation', SmallInteger, nullable = False, default = 0).create(getGoodsItem(meta))
    if not 'reverseoperation' in getStornoGoodsItem(meta).columns: 
        Column('reverseoperation', SmallInteger, nullable = False, default = 0).create(getStornoGoodsItem(meta))
    if not 'reverseoperation' in getMoneyItem(meta).columns: 
        Column('reverseoperation', SmallInteger, nullable = False, default = 0).create(getMoneyItem(meta))
    if not 'sumcash' in getDocument(meta).columns: 
        Column('sumcash', Numeric(precision = 13, scale = 2), nullable = False, default = 0).create(getDocument(meta))

    migrate.changeset.constraint.ForeignKeyConstraint([getDocument(meta).c.doctype], [documenttype.c.doctype], name = 'documenttypefk').create()
        
    if isinstance(migrate_engine.connect().dialect, MySQLDialect):
        createTriggers(migrate_engine)


def downgrade(migrate_engine):
    # Operations to reverse the above upgrade go here.
    
    meta = MetaData(bind = migrate_engine)
    
    moneyitemtype = getMoneyItemType(meta)
    goodsitemtype = getGoodsItemType(meta)
    documenttype = getDocumentType(meta)

    if isinstance(migrate_engine.connect().dialect, MySQLDialect):
        dropTriggers(migrate_engine)
    
    if 'documenttypefk' in [key.name for key in getDocument(meta).foreign_keys]:
        migrate.changeset.constraint.ForeignKeyConstraint([getDocument(meta).c.doctype], [documenttype.c.doctype], name = 'documenttypefk').drop()   
        
    if 'sumcash' in getDocument(meta).columns: 
        getDocument(meta).c.sumcash.drop()
    if 'reverseoperation' in getGoodsItem(meta).columns: 
        getGoodsItem(meta).c.reverseoperation.drop()
    if 'reverseoperation' in getStornoGoodsItem(meta).columns: 
        getStornoGoodsItem(meta).c.reverseoperation.drop()  
    if 'reverseoperation' in getMoneyItem(meta).columns:   
        getMoneyItem(meta).c.reverseoperation.drop()    
    
    if moneyitemtype.exists(meta.bind):
        moneyitemtype.drop()
    if goodsitemtype.exists(meta.bind):
        goodsitemtype.drop()
    if documenttype.exists(meta.bind):
        documenttype.drop()
        
    