from sqlalchemy import *
from migrate import *
import migrate.changeset

def getCardItem(meta):
    return Table('carditem', meta, autoload = True)

def getCouponItem(meta):
    return Table('couponitem', meta, autoload = True)

def getDocument(meta):
    return Table('document', meta, autoload = True)

def upgrade(migrate_engine):
    meta = MetaData(bind = migrate_engine)

    table = getCouponItem(meta)

    if not 'type' in table.columns:
        Column('type', Integer, nullable = False).create(table)
    if not 'status' in table.columns:
        Column('status', Integer, nullable = True).create(table)
    if not 'datebegin' in table.columns:
        Column('datebegin', DateTime, nullable = True).create(table)
    if not 'documentid' in table.columns:
        Column('documentid', Integer, ForeignKey(getDocument(meta).c.documentid, name ='documentidfk', ondelete = 'CASCADE'), nullable = False).create(table)

    for fkey in ['carditemfk', 'carditemfk_1']:
        if fkey in [key.name for key in table.foreign_keys]:
            migrate.changeset.constraint.ForeignKeyConstraint([table.c.carditemid], [getCardItem(meta).c.carditemid], name = fkey).drop()

    if 'carditemfk' in [ind.name for ind in table.indexes]:
        Index('carditemfk', table.c.carditemid, unique=False).drop()

    if 'carditemid' in table.columns:
        table.c.carditemid.alter(nullable = True)

    if not 'documentidfk' in [key.name for key in table.foreign_keys]:
        migrate.changeset.constraint.ForeignKeyConstraint([table.c.documentid], [getDocument(meta).c.documentid], name = 'documentidfk', ondelete = 'CASCADE').create()

def downgrade(migrate_engine):
    meta = MetaData(bind = migrate_engine)
    table = getCouponItem(meta)

    if 'type' in table.columns:
        table.c.type.drop()
    if 'status' in table.columns:
        table.c.status.drop()
    if 'datebegin' in table.columns:
        table.c.datebegin.drop()

    if 'documentidfk' in [key.name for key in table.foreign_keys]:
        migrate.changeset.constraint.ForeignKeyConstraint([table.c.documentid], [getDocument(meta).c.documentid], name = 'documentidfk').drop()

    if 'documentid' in table.columns:
        table.c.documentid.drop()

    if 'carditemid' in table.columns:
        table.c.carditemid.alter(nullable = False)

    if not 'carditemfk' in [key.name for key in table.foreign_keys]:
        migrate.changeset.constraint.ForeignKeyConstraint([table.c.carditemid], [getCardItem(meta).c.carditemid], name = 'carditemfk', ondelete = 'CASCADE').create()