from sqlalchemy import *
from migrate import *
from documents.versions import dump

expDateColumn = "expirationdate"

def getDraftBeerItemTable(meta):
    return Table('draftbeeritem', meta,
        Column('draftbeeritemid', Integer, nullable=False, primary_key=True, autoincrement=True),
        Column('cashcode', String(255), nullable=False),
        Column('markingcode', String(255), nullable=False),
        Column('barcode', String(255), nullable=False),
        Column('name', String(255), nullable=False),
        Column('operationtime', DateTime, nullable=False),
        Column('quant', Numeric(precision=13, scale=3), nullable=False),
        Column('operationid', String(255), nullable=False),
        Column('status', String(255), nullable=False),
        mysql_engine='InnoDB'
    )

def getGoodsItemTables(meta):
    goodsitem = Table('goodsitem', meta, autoload=True)
    stornogoodsitem = Table('stornogoodsitem', meta, autoload=True)
    return [goodsitem, stornogoodsitem]

def getDocType(meta):
    return Table('documenttype', meta, autoload=True)

def getGoodsItemType(meta):
    return Table('goodsitemtype', meta, autoload=True)

def insertIfNotExist(table, params):
    if not select([table.c.name]).where(and_(table.c.doctype == params['doctype'], table.c.name == params['name'])).execute().first():
        table.insert().values(params).execute()

def upgrade(migrate_engine):
    meta = MetaData(bind=migrate_engine)
    table = getDraftBeerItemTable(meta)
    dump.store(migrate_engine)

    if not table.exists(meta.bind):
        table.create()

    for goodsTable in getGoodsItemTables(meta):
        if expDateColumn not in goodsTable.columns:
            Column(expDateColumn, DateTime, nullable=True).create(goodsTable)

    insertIfNotExist(getDocType(meta), dict(doctype = 29, moneyfactor =  0, name = 'Постановка кега на кран'))
    insertIfNotExist(getGoodsItemType(meta), dict(doctype =  29, factor =  0, name = 'Постановка кега на кран', reverseoperation = 0))

def downgrade(migrate_engine):
    meta = MetaData(bind=migrate_engine)
    table = getDraftBeerItemTable(meta)

    if table.exists(meta.bind):
        table.drop()

    for goodsTable in getGoodsItemTables(meta):
        if expDateColumn in goodsTable.columns:
            Column(expDateColumn, DateTime, nullable=True).drop(goodsTable)
