from sqlalchemy import *
from migrate import *

def getTable(meta):
    return Table('cashierstats', meta,
        Column('cashierstatsid', Integer, nullable=False, primary_key=True, autoincrement=True),
        Column('workshiftid', Integer, nullable=False),
        Column('scode', String(30), nullable=True),
        Column('cashcode', String(30), nullable=False),
        Column('date_beg', DateTime, nullable=False),
        Column('shopcode', String(30), nullable=False),
        Column('sumsale', Numeric(precision=13, scale=2), nullable=False),
        Column('sumgain', Numeric(precision=13, scale=2), nullable=False),
        Column('sumsalecash', Numeric(precision=13, scale=2), nullable=False),
        Column('sumsalenoncash', Numeric(precision=13, scale=2), nullable=False),
        Column('sumsaleother', Numeric(precision=13, scale=2), nullable=False),
        Column('sumrefund', Numeric(precision=13, scale=2), nullable=False),
        Column('sumrefundcash', Numeric(precision=13, scale=2), nullable=False),
        Column('sumrefundnoncash', Numeric(precision=13, scale=2), nullable=False),
        Column('countsale', Integer, nullable=False),
        Column('countrefund', Integer, nullable=False),
        Column('countstorno', Integer, nullable=False),
        Column('countdisc', Integer, nullable=False),
        Column('sumstorno', Numeric(precision=13, scale=2), nullable=False),
        Column('timeaddposition', Numeric(precision=13, scale=2), nullable=False),
        Column('countposition', Integer, nullable=False),
        mysql_engine='InnoDB'
    )

idxName = 'workshiftuseridx'

def getIndex(tbl):
    return Index(idxName, tbl.c.workshiftid, tbl.c.scode, unique=True)

def upgrade(migrate_engine):
    meta = MetaData(bind=migrate_engine)
    statsTable = getTable(meta)
    if not statsTable.exists(meta.bind):
        statsTable.create()
        if idxName not in [ind.name for ind in statsTable.indexes]:
            getIndex(statsTable).create()

def downgrade(migrate_engine):
    meta = MetaData(bind=migrate_engine)
    statsTable = getTable(meta)
    if statsTable.exists(meta.bind):
        if idxName in [ind.name for ind in statsTable.indexes]:
            getIndex(statsTable).drop()
        statsTable.drop()

    